@@ -29,7 +29,6 @@
 #include <QSet>
 #include <PackageKit/Transaction>
 #include <AppstreamQt/database.h>
-#include <functional>
 
 class AppstreamReviews;
 class AppPackageKitResource;
@@ -64,6 +63,8 @@ class DISCOVERCOMMON_EXPORT PackageKitBackend : public AbstractResourcesBackend
         QVector<AppPackageKitResource*> extendedBy(const QString& id) const;
         void fetchUpdates();
 
+        void resolvePackages(const QStringList &packageNames);
+
     public Q_SLOTS:
         void transactionCanceled(Transaction* t);
         void removeTransaction(Transaction* t);
